@@ -23,15 +23,17 @@
 from future_builtins import filter, map
 
 import os  # @UnusedImport
-import re  # @UnusedImport
 import stat  # @UnusedImport
 import sys
+import re
 
 from duplicity.path import *  # @UnusedWildImport
 from duplicity import log  # @Reimport
 from duplicity import globals  # @Reimport
 from duplicity import diffdir
 from duplicity import util  # @Reimport
+from duplicity.globmatch import GlobbingError, FilePrefixError, \
+    path_matches_glob
 
 """Iterate exactly the requested files in a directory
 
@@ -46,16 +48,6 @@ class SelectError(Exception):
     pass
 
 
-class FilePrefixError(SelectError):
-    """Signals that a specified file doesn't start with correct prefix"""
-    pass
-
-
-class GlobbingError(SelectError):
-    """Something has gone wrong when parsing a glob string"""
-    pass
-
-
 class Select:
     """Iterate appropriate Paths in given directory
 
@@ -133,8 +125,7 @@ class Select:
                              util.escape(fullpath))
             except OSError:
                 log.Warn(_("Error accessing possibly locked file %s") % util.ufn(fullpath),
-                         log.WarningCode.cannot_stat,
-                         util.escape(fullpath))
+                         log.WarningCode.cannot_stat, util.escape(fullpath))
             return None
 
         def diryield(path):
@@ -151,20 +142,24 @@ class Select:
             for filename in robust.listpath(path):
                 new_path = robust.check_common_error(
                     error_handler, Path.append, (path, filename))
-                # make sure file is read accessible
-                if (new_path and new_path.type in ["reg", "dir"]
-                        and not os.access(new_path.name, os.R_OK)):
-                    log.Warn(_("Error accessing possibly locked file %s") % util.ufn(new_path.name),
-                             log.WarningCode.cannot_read,
-                             util.escape(new_path.name))
-                    if diffdir.stats:
-                        diffdir.stats.Errors += 1
-                    new_path = None
-                elif new_path:
+                if new_path:
                     s = self.Select(new_path)
-                    if s == 1:
+                    if (new_path.type in ["reg", "dir"] and
+                        not os.access(new_path.name, os.R_OK)) and \
+                            (s == 1 or s == 2):
+                        # Path is a file or folder that cannot be read, but
+                        # should be included or scanned.
+                        log.Error(_("Error accessing possibly locked file %s") %
+                                  util.ufn(new_path.name),
+                                  log.WarningCode.cannot_read,
+                                  util.escape(new_path.name))
+                        if diffdir.stats:
+                            diffdir.stats.Errors += 1
+                    elif s == 1:
+                        # Should be included
                         yield (new_path, 0)
                     elif s == 2 and new_path.isdir():
+                        # Is a directory that should be scanned
                         yield (new_path, 1)
 
         if not path.type:
@@ -302,7 +297,7 @@ pattern (such as '**') which matches the base directory.""") %
             log.FatalError(_("Fatal Error while processing expression\n"
                              "%s") % exc, log.ErrorCode.globbing_error)
         else:
-            raise
+            raise  # pylint: disable=misplaced-bare-raise
 
     def parse_last_excludes(self):
         """Exit with error if last selection function isn't an exclude"""
@@ -540,57 +535,22 @@ probably isn't what you meant.""") %
         """
         # Internal. Used by glob_get_sf and unit tests.
 
-        match_only_dirs = False
-
-        if glob_str != "/" and glob_str[-1] == "/":
-            match_only_dirs = True
-            # Remove trailing / from directory name (unless that is the entire
-            # string)
-            glob_str = glob_str[:-1]
+        ignore_case = False
 
         if glob_str.lower().startswith("ignorecase:"):
-            re_comp = lambda r: re.compile(r, re.I | re.S)
-            glob_str = glob_str[len("ignorecase:"):]
-        else:
-            re_comp = lambda r: re.compile(r, re.S)
-
-        # matches what glob matches and any files in directory
-        glob_comp_re = re_comp("^%s($|/)" % self.glob_to_re(glob_str))
-
-        if glob_str.find("**") != -1:
-            glob_str = glob_str[:glob_str.find("**") + 2]  # truncate after **
-
-        scan_comp_re = re_comp("^(%s)$" %
-                               "|".join(self.glob_get_prefix_res(glob_str)))
-
-        def include_sel_func(path):
-            if match_only_dirs and not path.isdir():
-                # If the glob ended with a /, only match directories
-                return None
-            elif glob_comp_re.match(path.name):
-                return 1
-            elif scan_comp_re.match(path.name):
-                return 2
-            else:
-                return None
-
-        def exclude_sel_func(path):
-            if match_only_dirs and not path.isdir():
-                # If the glob ended with a /, only match directories
-                return None
-            elif glob_comp_re.match(path.name):
-                return 0
-            else:
-                return None
+            # Glob string starts with ignorecase, so remove that from the
+            # string and change it to lowercase.
+            glob_str = glob_str[len("ignorecase:"):].lower()
+            ignore_case = True
 
         # Check to make sure prefix is ok
-        if not include_sel_func(self.rootpath):
+        if not path_matches_glob(self.rootpath, glob_str, include=1):
             raise FilePrefixError(glob_str)
 
-        if include:
-            return include_sel_func
-        else:
-            return exclude_sel_func
+        def sel_func(path):
+            return path_matches_glob(path, glob_str, include, ignore_case)
+
+        return sel_func
 
     def exclude_older_get_sf(self, date):
         """Return selection function based on files older than modification date """
@@ -609,62 +569,3 @@ probably isn't what you meant.""") %
         sel_func.exclude = True
         sel_func.name = "Select older than %s" % (date,)
         return sel_func
-
-    def glob_get_prefix_res(self, glob_str):
-        """Return list of regexps equivalent to prefixes of glob_str"""
-        # Internal. Used by glob_get_normal_sf.
-        glob_parts = glob_str.split("/")
-        if "" in glob_parts[1:-1]:
-            # "" OK if comes first or last, as in /foo/
-            raise GlobbingError("Consecutive '/'s found in globbing string "
-                                + glob_str)
-
-        prefixes = ["/".join(glob_parts[:i + 1]) for i in range(len(glob_parts))]
-        # we must make exception for root "/", only dir to end in slash
-        if prefixes[0] == "":
-            prefixes[0] = "/"
-        return map(self.glob_to_re, prefixes)
-
-    def glob_to_re(self, pat):
-        """Returned regular expression equivalent to shell glob pat
-
-        Currently only the ?, *, [], and ** expressions are supported.
-        Ranges like [a-z] are also currently unsupported.  There is no
-        way to quote these special characters.
-
-        This function taken with minor modifications from efnmatch.py
-        by Donovan Baarda.
-
-        """
-        # Internal. Used by glob_get_normal_sf, glob_get_prefix_res and unit tests.
-        i, n, res = 0, len(pat), ''
-        while i < n:
-            c, s = pat[i], pat[i:i + 2]
-            i = i + 1
-            if s == '**':
-                res = res + '.*'
-                i = i + 1
-            elif c == '*':
-                res = res + '[^/]*'
-            elif c == '?':
-                res = res + '[^/]'
-            elif c == '[':
-                j = i
-                if j < n and pat[j] in '!^':
-                    j = j + 1
-                if j < n and pat[j] == ']':
-                    j = j + 1
-                while j < n and pat[j] != ']':
-                    j = j + 1
-                if j >= n:
-                    res = res + '\\['  # interpret the [ literally
-                else:
-                    # Deal with inside of [..]
-                    stuff = pat[i:j].replace('\\', '\\\\')
-                    i = j + 1
-                    if stuff[0] in '!^':
-                        stuff = '^' + stuff[1:]
-                    res = res + '[' + stuff + ']'
-            else:
-                res = res + re.escape(c)
-        return res
